# -*- mode:perl; fill-column:80 -*-
use utf8;

use 5.016;
use warnings;
use strict;

use POSIX qw/floor/;

our @codemap;
our %codemap;
do "./trianguish-codes.pl" or die;

my $halved_x_adj = 0;
my $y_adj = 0;

@ARGV and $ARGV[0] =~ /-?\d+/ and $halved_x_adj = shift;
@ARGV and $ARGV[0] =~ /-?\d+/ and $y_adj = shift;

# The text-based format uses the following coordinate system:
#
# y x0 1 2 3 4 5
# 0 ┌┴┐ ┌┴┐ ┌┴┐
# 0   └┬┘ └┬┘ └┬┘
# 1   ┌┴┐ ┌┴┐ ┌┴┐
# 1 └┬┘ └┬┘ └┬┘
# 2 ┌┴┐ ┌┴┐ ┌┴┐
# 2   └┬┘ └┬┘ └┬┘
# 3   ┌┴┐ ┌┴┐ ┌┴┐
# 3 └┬┘ └┬┘ └┬┘
#
# but the JS interpreter uses a different coordinate system:
#       030 130 230
#       021 121 221
#     020 120 220
#     011 111 211
#   010 110 210
#   001 101 201
# 000 100 200
#
# Conversions (text-based y, x to JS q, r, s):
#
# q = (x + y + 1) >> 1
# r = -y
# s = (x + y + 1) & 1

my $odd_x = 0;
my $up_pointing = 1;
my $y = 0;
my $x_max = 0;

my @program;

while(<>) {
    next if /^$/;

    /^(  )?(\S\S\S(?: \S\S\S)*)\s*$/ or die
        "Input line is not in the expected format";
    !$1 == !$odd_x or die
        "Input line is not in the expected position";

    my $shifted_line = $2;
    my $x = $odd_x ? -1 : -2;
    $program[$y][$x += 2] = $_ for split / /, $shifted_line;
    $x > $x_max and $x_max = $x;

    $odd_x ^= 1 if $up_pointing;
    $up_pointing = !$up_pointing;
    ++$y if $up_pointing;
}

my $first = 1;
while (my ($y, $line) = each @program) {
    while (my ($x, $op) = each @$line) {
        next if $op eq '...';

        print ";" unless $first;
        undef $first;

        my $ax = $x + $halved_x_adj * 2;
        my $ay = $y + $y_adj;

        my $q = floor(($ax + $ay + 1) / 2);
        my $r = -$ay;
        my $s = ($ax + $ay + 1) & 1;
        $s or $op = reverse $op;
        print "$q $r $s ", $codemap{$op};
    }
}
print "\n";
