# -*- mode:perl; fill-column:80 -*-
use utf8;

use 5.016;
use warnings;
use strict;

our @codemap;
our %codemap;
do "./trianguish-codes.pl" or die;

# The text-based format uses the following coordinate system:
#
# y x0 1 2 3 4 5
# 0 ┌┴┐ ┌┴┐ ┌┴┐
# 0   └┬┘ └┬┘ └┬┘
# 1   ┌┴┐ ┌┴┐ ┌┴┐
# 1 └┬┘ └┬┘ └┬┘
# 2 ┌┴┐ ┌┴┐ ┌┴┐
# 2   └┬┘ └┬┘ └┬┘
# 3   ┌┴┐ ┌┴┐ ┌┴┐
# 3 └┬┘ └┬┘ └┬┘
#
# but the JS interpreter uses a different coordinate system:
#       030 130 230
#       021 121 221
#     020 120 220
#     011 111 211
#   010 110 210
#   001 101 201
# 000 100 200
#
# Conversions (from JS q, r, s to text-based y, x):
#
# y = -r
# x = 2q + s + r + 1

my %program;

my ($min_y, $max_y, $min_x, $max_x);

while (<>) {
    /^$/ and next;
    /^(-?\d+) (-?\d+) ([01]) (\d+)\s*$/
        or die 'Input lines should be in the format "q r s op"';
    my $y = -$2;
    my $x = 2*$1 + $2 + $3;
    $program{$y}{$x} = $3 ? $codemap[$4] : reverse $codemap[$4];
    $min_y //= $y; $min_y > $y and $min_y = $y;
    $min_x //= $x; $min_x > $x and $min_x = $x;
    $max_y //= $y; $max_y < $y and $max_y = $y;
    $max_x //= $x; $max_x < $x and $max_x = $x;
}

# parity of x, y is important
if ($min_x % 2) {
    --$min_x;
}

my @program;
for my $y ($min_y .. $max_y) {
    my $line = [];
    for my $x ($min_x .. $max_x) {
        push @$line, $program{$y}{$x} // "...";
    }
    push @program, $line;
}

output_text_grid(\*STDOUT, $max_x - $min_x, \@program);
