# -*- mode:perl; fill-column:80 -*-

use 5.016;
use warnings;
use strict;

# The various codes used by Trianguish, in numerical and text form.
#
# The text codes are for upwards-pointing triangles ┴.  For downwards-pointing
# triangles, you have to reverse the order of characters in the string.
our @codemap;

# Not all the codes are listed here: rotated and reflected codes are produced by
# permuting the characters of the code in a predictable way:
#
# x+0 ABC
# x+1 CBA
# x+2 CAB
# x+3 ACB
# x+4 BCA
# x+5 BAC
sub permute_codemap {
    my $i = shift;
    $codemap[$i] =~ /^(.)(.)(.)$/;
    my $all_different = $1 ne $2 && $2 ne $3 && $3 ne $1;
    $codemap[$i+0] = "$1$2$3";
    $codemap[$i+1] = "$3$2$1";
    $codemap[$i+2] = "$2$3$1";
    $codemap[$i+3] = "$2$1$3";
    $codemap[$i+4] = "$3$1$2";
    $codemap[$i+5] = "$1$3$2";
}

$codemap[  0] = '...'; permute_codemap   0;
$codemap[ 18] = '000'; $codemap[ 24] = '111'; $codemap[ 30] = '---';
permute_codemap    18; permute_codemap    24; permute_codemap    30;
$codemap[ 36] = 'i-o'; permute_codemap  36;
$codemap[ 54] = 'ioo'; permute_codemap  54;
$codemap[ 72] = 'iTo'; permute_codemap  72;
$codemap[ 90] = 'iSo'; permute_codemap  90;
$codemap[108] = 'iNo'; permute_codemap 108;
$codemap[126] = 'ii+'; permute_codemap 126;
$codemap[144] = 'ii*'; permute_codemap 144;
$codemap[162] = 'iCo'; permute_codemap 162;
$codemap[180] = 'iIo'; permute_codemap 180;
$codemap[198] = 'bIo'; $codemap[204] = 'bOi'; $codemap[210] = 'bRo';
permute_codemap   198; permute_codemap   204; permute_codemap   210;

our %codemap;
defined $codemap[$_] and $codemap{$codemap[$_]} //= $_ for 0..$#codemap;

sub output_text_grid {
    my $handle = shift;
    my $x_max = shift;
    my $aref = shift;
    my $numerical = shift;

    my $odd_x = 0;
    for my $line (@$aref) {
        for my $parity ($odd_x, 1^$odd_x) {
            $parity and $handle->print("  ");
            for my $x (0 .. $x_max) {
                $x % 2 == $parity or next;
                my $val = $line->[$x];
                if (!defined $val) {
                    $handle->print("### ");
                } elsif ($numerical) {
                    $handle->printf("%03d ", $val);
                } else {
                    $handle->printf("%3.3s ", $val);
                }
            }
            $handle->print("\n");
        }
        $odd_x = !$odd_x;
    }
}

1;
